#include "StdAfx.h"
#include "../common/common.h"
#include "../display/common.h"
#include "../input/common.h"

#include ".\terrain.h"

CTerrain::CTerrain(void)
{
	m_iTerrainTilesX = 0;
	m_iTerrainTilesY = 0;
	m_iTerrainVerticesX = 0;
	m_iTerrainVerticesY = 0;
	m_iTerrainMethod = TM_HEIGHTMAP;
	m_pTerrainIB = NULL;
	m_pTerrainSurface = NULL;
	m_pHeightMap = NULL;
	m_arTerrain = NULL;
	m_dwFlags |= TF_USEQUADS;
}

CTerrain::~CTerrain(void)
{
	int i;
	if(m_arTerrain)
	{
		for(i = 0; i < m_iTerrainVerticesX; i++)
		{
			SAFE_DELETE_ARRAY(m_arTerrain[i]);
		}
		SAFE_DELETE_ARRAY(m_arTerrain);
	}
}

HRESULT			CTerrain::InternalInit()
{
	DWORD dwRet = S_FALSE;
	DWORD dwTerrainIBSize, dwIndicesCount;
	IDisplay *pDis;
	CreateDisplayObject(DISIID_IDisplay, (void**) &pDis);
	//
	// Create font 
	if(S_OK == (dwRet = CreateDisplayObject(DISIID_I3DFont, (void**) &m_pDebugFont)))
	{
		m_pDebugFont->CreateFont(15, 8, "Arial", TRUE, D3DCOLOR_ARGB(255,150,255,100));
	}
	else
	{
		TRACE("Cannot create font: %d", dwRet);		
		return dwRet;
	}
	//
	// Create common index buffer for all quad leaves
	dwRet = CreateDisplayObject(DISIID_IIndexBuffer, (void**) &m_pTerrainIB);
	if(dwRet == S_OK)
	{	
		dwTerrainIBSize = LEAF_I_SIZE * LEAF_I_SIZE * 6 * sizeof(WORD);
		dwIndicesCount = LEAF_I_SIZE * LEAF_I_SIZE * 6;
		dwRet = m_pTerrainIB->Create(dwTerrainIBSize, D3DUSAGE_WRITEONLY, D3DFMT_INDEX16, D3DPOOL_DEFAULT);
		if(dwRet != S_OK)
		{
			XException exp("Cannot create IB for terrain!", dwRet);
			THROW(exp);
		}
	}

	//
	// Load terrain texture
	dwRet = CreateDisplayObject(DISIID_ITexture, (void**)&m_pTerrainSurface);
	if(dwRet == S_OK)
	{
		dwRet = m_pTerrainSurface->LoadTextureFromFileEx("grass64.bmp", 6, pDis->GetTextureFormat());
		if(dwRet != S_OK)
		{
			XException exp("Cannot load texture for terrain surface!", dwRet);
			THROW(exp);
		} 
	}		

	SAFE_RELEASE(pDis);
	
	return dwRet;
}

int		CTerrain::ComputeNormals()
{
	float z1, z2;
	D3DXVECTOR3	v1, v2, v0;
	D3DXVECTOR3	s1, s2, n0;
	int x, y;
	//
	// Check terrain validity
	if(!m_arTerrain)
	{
		XException exp("Terrain is not initialized. You cannot apply any algorithm.");
		THROW(exp);
	}
	for(y = 0; y < m_iTerrainVerticesY; y++)
	{
		for(x = 0; x < m_iTerrainVerticesX; x++)
		{
			v0 = m_arTerrain[x][y].vecPos;
			if(x < m_iTerrainTilesX)
			{
				z1 = m_arTerrain[x+1][y].vecPos.z;				
			}
			else
			{
				z1 = m_arTerrain[x-1][y].vecPos.z;				
			}
			
			if(y < m_iTerrainTilesY)
			{
				z2 = m_arTerrain[x][y+1].vecPos.z;				
			}
			else
			{
				z2 = m_arTerrain[x][y-1].vecPos.z;				
			}
			v1 = D3DXVECTOR3(float(x+1), float(y), z1);
			v2 = D3DXVECTOR3(float(x), float(y+1), z2);
			s1 = v0 - v1;
			s2 = v0 - v2;		
			
			// create normal vector
			D3DXVec3Cross(&n0, &s1, &s2);
			D3DXVec3Normalize(&n0, &n0);
			m_arTerrain[x][y].vecNormal = n0;
		}
	}	
	return S_OK;
}


int		CTerrain::CreateQuadTree()
{
	//
	// Check terrain validity
	if(!m_arTerrain)
	{
		XException exp("Terrain is not initialized. You cannot apply any algorithm.");
		THROW(exp);
	}
	if(m_iTerrainVerticesX >= LEAF_I_SIZE && m_iTerrainVerticesY >= LEAF_I_SIZE)
	{
		m_pRoot = new QTNode;
		// Init ROOT
		m_pRoot->ntType = NODE_TYPE;
		m_pRoot->pVB = NULL;
		m_pRoot->arBounds[0].x = m_arTerrain[0][0].vecPos.x;
		m_pRoot->arBounds[0].y = m_arTerrain[0][0].vecPos.y;
		m_pRoot->arBounds[0].z = m_arTerrain[0][0].vecPos.z;
		m_pRoot->arBounds[1].x = m_arTerrain[m_iTerrainVerticesX-1][0].vecPos.x;
		m_pRoot->arBounds[1].y = m_arTerrain[m_iTerrainVerticesX-1][0].vecPos.y;
		m_pRoot->arBounds[1].z = m_arTerrain[m_iTerrainVerticesX-1][0].vecPos.z;
		m_pRoot->arBounds[2].x = m_arTerrain[0][m_iTerrainVerticesY-1].vecPos.x;
		m_pRoot->arBounds[2].y = m_arTerrain[0][m_iTerrainVerticesY-1].vecPos.y;
		m_pRoot->arBounds[2].z = m_arTerrain[0][m_iTerrainVerticesY-1].vecPos.z;
		m_pRoot->arBounds[3].x = m_arTerrain[m_iTerrainVerticesX-1][m_iTerrainVerticesY-1].vecPos.x;
		m_pRoot->arBounds[3].y = m_arTerrain[m_iTerrainVerticesX-1][m_iTerrainVerticesY-1].vecPos.y;
		m_pRoot->arBounds[3].z = m_arTerrain[m_iTerrainVerticesX-1][m_iTerrainVerticesY-1].vecPos.z;

		CreateNode(m_pRoot);
		//
		// Fill index buffer and leaves with data
		Restore();
	}
	else
	{
		return S_FALSE;
	}
	return S_OK;
}

HRESULT		CTerrain::GenerateTerrain(int x, int y, TERRAIN_METHOD tmMethod /*= TM_CIRCLE*/)
{
	DWORD dwRet = S_FALSE;
	int i = 0;
	//
	// Init common object (texture, font etc.)
	InternalInit();

	m_iTerrainTilesX = x;
	m_iTerrainTilesY = y;
	m_iTerrainVerticesX = x + 1;
	m_iTerrainVerticesY = y + 1;
	
	m_arTerrain = new VERTEX*[m_iTerrainVerticesX];
	for(i = 0; i < m_iTerrainVerticesX; i++)
	{
		m_arTerrain[i] = new VERTEX[m_iTerrainVerticesY];
	}
	//
	// Init basic terrain parameters
	for(y = 0; y < m_iTerrainVerticesY; y++)
	{
		for(x = 0; x < m_iTerrainVerticesX; x++)
		{				
			m_arTerrain[x][y].vecPos = D3DXVECTOR3(float(x), float(y), 0.0f);						
			m_arTerrain[x][y].dwDiffuse = D3DCOLOR_ARGB(128,128,255,128);
			m_arTerrain[x][y].tu1 = (x % 2) ? 1.0f : 0.0f;
			m_arTerrain[x][y].tv1 = (y % 2) ? 1.0f : 0.0f;	
			m_arTerrain[x][y].vecNormal = D3DXVECTOR3(0.0f, 0.0f, 1.0f);			
		}
	}
	//
	// Apply circle algorithm on the terrain
	CircleAlgorithm();
	ComputeNormals();
	//
	// Create quad tree
	CreateQuadTree();
	
	return dwRet;
}

HRESULT		CTerrain::GenerateTerrainFromFile(LPCSTR szFile)
{
	DWORD dwRet = S_FALSE;
	int i = 0, x, y;	
	//
	// Init common object (texture, font etc.)
	InternalInit();
	//
	// Load heightmap texture
	dwRet = CreateDisplayObject(DISIID_ITexture, (void**)&m_pHeightMap);
	if(dwRet == S_OK)
	{
		dwRet = m_pHeightMap->LoadTextureFromFile(szFile);
		if(dwRet != S_OK)
		{
			XException exp("Cannot load texture for heightmap!", dwRet);
			THROW(exp);
		}                
		if(m_pHeightMap->GetFormat() != D3DFMT_A8R8G8B8 && m_pHeightMap->GetFormat() != D3DFMT_X8R8G8B8)
		{
			XException exp("Invalid heightmap format! Must be 32-bit with alpha channel.");
			THROW(exp);
		}
		//set dim of the terrain according dim of the height map
		m_iTerrainTilesX = m_pHeightMap->Width() - 1;
		m_iTerrainTilesY = m_pHeightMap->Height() - 1;
		m_iTerrainVerticesX = m_pHeightMap->Width();
		m_iTerrainVerticesY = m_pHeightMap->Height();
	}				
	m_arTerrain = new VERTEX*[m_iTerrainVerticesX];
	for(i = 0; i < m_iTerrainVerticesX; i++)
	{
		m_arTerrain[i] = new VERTEX[m_iTerrainVerticesY];
	}
	//
	// Open heightmap
	D3DLOCKED_RECT lr;
	m_pHeightMap->GetTexture()->LockRect(0, &lr, NULL, D3DLOCK_READONLY);
	TEXTURE_PIXEL * data = (TEXTURE_PIXEL*)lr.pBits;
		
	int p = lr.Pitch/4;
	// Init basic terrain parameters
	for(y = 0; y < m_iTerrainVerticesY; y++)
	{
		for(x = 0; x < m_iTerrainVerticesX; x++)
		{		
			TEXTURE_PIXEL tp = data[y*p + x];

			m_arTerrain[x][y].vecPos = D3DXVECTOR3(float(x), float(y), float(tp.a)/255.0f*20.0f-10.0f);						
			m_arTerrain[x][y].dwDiffuse = D3DCOLOR_ARGB(128, tp.r, tp.g, tp.b);
			m_arTerrain[x][y].tu1 = (x % 2) ? 1.0f : 0.0f;
			m_arTerrain[x][y].tv1 = (y % 2) ? 1.0f : 0.0f;	
			m_arTerrain[x][y].vecNormal = D3DXVECTOR3(0.0f, 0.0f, 1.0f);			
		}
	}
	m_pHeightMap->GetTexture()->UnlockRect(0);

	ComputeNormals();
	//
	// Create quad tree
	CreateQuadTree();

	return dwRet;
}

// Fills queue of the visible quads
HRESULT		CTerrain::CullTerrain(QTNode *pNode, CLIPVOLUME& cv)
{
	DWORD zones[4] = {0, 0, 0, 0};
	FLOAT x, y, z;
	float temp;
	
	for(int i = 0; i < 4; i++)
	{
		x = pNode->arBounds[i].x;
		y = pNode->arBounds[i].y;
		z = pNode->arBounds[i].z;

		temp = cv.pNear.a * x + cv.pNear.b * y + cv.pNear.c * z + cv.pNear.d;
		if (temp > CULL_TOLERANCE) {
			zones[i] |= 0x01;			
		}
		else {
			temp = cv.pFar.a * x + cv.pFar.b * y + cv.pFar.c * z + cv.pFar.d;
			if (temp > CULL_TOLERANCE) {
				zones[i] |= 0x02;
			}
		}

		temp = cv.pLeft.a * x + cv.pLeft.b * y + cv.pLeft.c * z + cv.pLeft.d;
		if (temp > CULL_TOLERANCE) {
			zones[i] |= 0x04;
		}
		else {
			temp = cv.pRight.a * x + cv.pRight.b * y + cv.pRight.c * z + cv.pRight.d;
			if (temp > CULL_TOLERANCE) {
				zones[i] |= 0x08;
			}
		}

		temp = cv.pTop.a * x + cv.pTop.b * y + cv.pTop.c * z + cv.pTop.d;
		if (temp > CULL_TOLERANCE+2.0f) {
			zones[i] |= 0x10;
		}
		else 
		{
			temp = cv.pBottom.a * x + cv.pBottom.b * y + cv.pBottom.c * z + cv.pBottom.d;
			if (temp > CULL_TOLERANCE) {
				zones[i] |= 0x20;
			}
		}
	}

	//if all of the corners are outside of the boundaries
	// this node is excluded, so stop traversing
	DWORD res = zones[0] & zones[1] & zones[2] & zones[3];
	if(res)
	{
		return -1;
	}

	// if this is a leaf add the triangle lists to the render queue
	if (pNode->ntType == LEAF_TYPE)
	{
		pNode->bVis = TRUE;
		m_arVisQuads.push_back(pNode);
		return 1;
	}
	else 
	{
		//this is not a leaf traverse deeper
		for(i = 0; i < 4; i++) {
			CullTerrain(pNode->pBranches[i], cv);
		}		
	}
	return 0;
}


HRESULT		CTerrain::Render()
{
	IDisplay *pDis;
	CLIPVOLUME cv;
	PIXEL v;
	int i;
	char szInfo[255];
	CreateDisplayObject(DISIID_IDisplay, (void**) &pDis);

	if(m_dwFlags & TF_DRAWQUADSMAP)
	{
		for(i = 0; i < (int)m_arLeaves.size(); i++)
		{
			((QTNode*)m_arLeaves[i])->bVis = FALSE;
		}
	}

	m_arVisQuads.clear();
	pDis->GetCamera()->GetClipVolume(cv);
	CullTerrain(m_pRoot, cv);

	pDis->GetDevice()->SetVertexShader(VERTEXFORMAT);			
	pDis->GetDevice()->SetIndices(m_pTerrainIB->GetBuffer(), 0);
	pDis->GetDevice()->SetTexture(0, m_pTerrainSurface->GetTexture());

	if(m_dwFlags & TF_USEQUADS)
	{
		for(i = 0; i < (int)m_arVisQuads.size(); i++)
		{
			pDis->GetDevice()->SetStreamSource(0, ((QTNode*)m_arVisQuads[i])->pVB->GetBuffer(), sizeof(VERTEX));
			pDis->GetDevice()->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, LEAF_V_SIZE*LEAF_V_SIZE, 0, LEAF_I_SIZE*LEAF_I_SIZE * 2);
		}
		sprintf(szInfo, "Using quad tree culling\nTotal leaves: %d\nTotal faces: %d", (int)m_arVisQuads.size(), (int)m_arVisQuads.size() * LEAF_I_SIZE * LEAF_I_SIZE * 2);
	}
	else
	{
		for(i = 0; i < (int)m_arLeaves.size(); i++)
		{
			pDis->GetDevice()->SetStreamSource(0, ((QTNode*)m_arLeaves[i])->pVB->GetBuffer(), sizeof(VERTEX));
			pDis->GetDevice()->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, LEAF_V_SIZE*LEAF_V_SIZE, 0, LEAF_I_SIZE*LEAF_I_SIZE * 2);
		}
		sprintf(szInfo, "Without quad tree culling\nTotal leaves: %d\nTotal faces: %d", (int)m_arLeaves.size(), (int)m_arLeaves.size() * LEAF_I_SIZE * LEAF_I_SIZE * 2);
	}	

	if(m_dwFlags & TF_DRAWQUADSMAP)
	{
		pDis->GetDevice()->SetVertexShader(PIXELFORMAT);			
		pDis->GetDevice()->SetRenderState(D3DRS_LIGHTING, FALSE);
		pDis->GetDevice()->SetTexture(0, NULL);
		for(i = 0; i < (int)m_arLeaves.size(); i++)
		{
			QTNode *pNode = (QTNode*)m_arLeaves[i];
			v.vecPos.x = pNode->arBounds[0].x/LEAF_I_SIZE + pDis->GetResolution()->x - m_iTerrainTilesX/LEAF_I_SIZE - 10;
			v.vecPos.y = pNode->arBounds[0].y/LEAF_I_SIZE + 10;
			v.vecPos.z = 0.0f;
			v.rhw = 1.0f;
			if(pNode->bVis)
			{
				v.dwDiffuse = D3DCOLOR_ARGB(255,255,255,0);				
			}
			else
			{
				v.dwDiffuse = D3DCOLOR_ARGB(255,128,128,128);
			}
			pDis->GetDevice()->DrawPrimitiveUP(D3DPT_POINTLIST, 1, &v, sizeof(PIXEL));			
		}
		pDis->GetDevice()->SetRenderState(D3DRS_LIGHTING, TRUE);
	}
	
	m_pDebugFont->Draw(szInfo, 0, 300);

	SAFE_RELEASE(pDis);
	return 0;
}

HRESULT		CTerrain::Restore()
{
	WORD *pIndices;
	DWORD dwRet = S_FALSE;
	int x, y, i = 0;

	if(m_arTerrain)
	{
		TRACE("Restoring terrain surface...");
		dwRet = m_pTerrainIB->GetBuffer()->Lock(0, 0, (BYTE**)&pIndices, 0);					
		for(y = 0; y < LEAF_I_SIZE; y++)
		{
			for(x = 0; x < LEAF_I_SIZE; x++)
			{
				pIndices[i + 0] = x + y * LEAF_V_SIZE;
				pIndices[i + 1] = (x+1) + y * LEAF_V_SIZE;
				pIndices[i + 2] = x + (y+1) * LEAF_V_SIZE;			
				i += 3;
				pIndices[i + 0] = (x+1) + y * LEAF_V_SIZE;
				pIndices[i + 1] = (x+1) + (y+1) * LEAF_V_SIZE;
				pIndices[i + 2] = x + (y+1) * LEAF_V_SIZE;			
				i += 3;				
			}
		}
		dwRet = m_pTerrainIB->GetBuffer()->Unlock();
		dwRet = FillLeaves();
	}
	return dwRet;
}

HRESULT		CTerrain::DestroyTerrain()
{
	DWORD dwRet = S_FALSE;

	DestroyNode(m_pRoot);

	SAFE_RELEASE(m_pHeightMap);
	SAFE_RELEASE(m_pTerrainSurface);
	SAFE_RELEASE(m_pTerrainIB);	
	SAFE_RELEASE(m_pDebugFont);

	return dwRet;
}

int		CTerrain::CreateNode(QTNode *pNode)
{
	UINT		uiWidth, uiHeight;		
	//
	// Check terrain validity
	if(!m_arTerrain)
	{
		XException exp("Terrain is not initialized. You cannot apply any algorithm.");
		THROW(exp);
	}
	if(pNode)
	{		
		//
		// Compute node width and height
		uiWidth =  int(pNode->arBounds[1].x - pNode->arBounds[0].x);
		uiHeight = int(pNode->arBounds[2].y - pNode->arBounds[0].y);
		//
		// Get node type
		if(uiWidth == LEAF_I_SIZE)
		{
			pNode->ntType = LEAF_TYPE;
			// Create vertex buffer
			if(S_OK == CreateDisplayObject(DISIID_IVertexBuffer, (void**) &pNode->pVB))
			{
				pNode->pVB->Create(LEAF_V_SIZE*LEAF_V_SIZE*sizeof(VERTEX), D3DUSAGE_WRITEONLY, VERTEXFORMAT, D3DPOOL_DEFAULT);				
				pNode->bVis = FALSE;
				m_arLeaves.push_back(pNode);
			}
			return 1;
		}
		else
		{
			pNode->ntType = NODE_TYPE;
			//
			// Create 4 children

			// First
			pNode->pBranches[0] = new QTNode;
			pNode->pBranches[0]->pVB = NULL;
			// Init new bounding box
			pNode->pBranches[0]->arBounds[0] = pNode->arBounds[0];

			pNode->pBranches[0]->arBounds[1].x = pNode->arBounds[1].x - uiWidth / 2;
			pNode->pBranches[0]->arBounds[1].y = pNode->arBounds[1].y;
			pNode->pBranches[0]->arBounds[1].z = m_arTerrain[int(pNode->pBranches[0]->arBounds[1].x)][int(pNode->pBranches[0]->arBounds[1].y)].vecPos.z;

			pNode->pBranches[0]->arBounds[2].x = pNode->arBounds[2].x;
			pNode->pBranches[0]->arBounds[2].y = pNode->arBounds[2].y - uiHeight / 2;
			pNode->pBranches[0]->arBounds[2].z = m_arTerrain[int(pNode->pBranches[0]->arBounds[2].x)][int(pNode->pBranches[0]->arBounds[2].y)].vecPos.z;

			pNode->pBranches[0]->arBounds[3].x = pNode->arBounds[3].x - uiWidth / 2;
			pNode->pBranches[0]->arBounds[3].y = pNode->arBounds[3].y - uiHeight / 2;
			pNode->pBranches[0]->arBounds[3].z = m_arTerrain[int(pNode->pBranches[0]->arBounds[3].x)][int(pNode->pBranches[0]->arBounds[3].y)].vecPos.z;
	
			// Second
			pNode->pBranches[1] = new QTNode;
			pNode->pBranches[1]->pVB = NULL;
			// Init new bounding box
			pNode->pBranches[1]->arBounds[0].x = pNode->arBounds[0].x + uiWidth / 2;
			pNode->pBranches[1]->arBounds[0].y = pNode->arBounds[0].y;			
			pNode->pBranches[1]->arBounds[0].z = m_arTerrain[int(pNode->pBranches[1]->arBounds[0].x)][int(pNode->pBranches[1]->arBounds[0].y)].vecPos.z;

			pNode->pBranches[1]->arBounds[1] = pNode->arBounds[1];
			
			pNode->pBranches[1]->arBounds[2].x = pNode->arBounds[2].x + uiWidth / 2;
			pNode->pBranches[1]->arBounds[2].y = pNode->arBounds[2].y - uiHeight / 2;			
			pNode->pBranches[1]->arBounds[2].z = m_arTerrain[int(pNode->pBranches[1]->arBounds[2].x)][int(pNode->pBranches[1]->arBounds[2].y)].vecPos.z;

			pNode->pBranches[1]->arBounds[3].x = pNode->arBounds[3].x;
			pNode->pBranches[1]->arBounds[3].y = pNode->arBounds[3].y - uiHeight / 2;
			pNode->pBranches[1]->arBounds[3].z = m_arTerrain[int(pNode->pBranches[1]->arBounds[3].x)][int(pNode->pBranches[1]->arBounds[3].y)].vecPos.z;

			// Third
			pNode->pBranches[2] = new QTNode;
			pNode->pBranches[2]->pVB = NULL;
			// Init new bounding box
			pNode->pBranches[2]->arBounds[0].x = pNode->arBounds[0].x;
			pNode->pBranches[2]->arBounds[0].y = pNode->arBounds[0].y + uiHeight / 2;			
			pNode->pBranches[2]->arBounds[0].z = m_arTerrain[int(pNode->pBranches[2]->arBounds[0].x)][int(pNode->pBranches[2]->arBounds[0].y)].vecPos.z;
			
			pNode->pBranches[2]->arBounds[1].x = pNode->arBounds[1].x - uiWidth / 2;
			pNode->pBranches[2]->arBounds[1].y = pNode->arBounds[1].y + uiHeight / 2;			
			pNode->pBranches[2]->arBounds[1].z = m_arTerrain[int(pNode->pBranches[2]->arBounds[1].x)][int(pNode->pBranches[2]->arBounds[1].y)].vecPos.z;

			pNode->pBranches[2]->arBounds[2] = pNode->arBounds[2];

			pNode->pBranches[2]->arBounds[3].x = pNode->arBounds[3].x - uiWidth / 2;
			pNode->pBranches[2]->arBounds[3].y = pNode->arBounds[3].y;			
			pNode->pBranches[2]->arBounds[3].z = m_arTerrain[int(pNode->pBranches[2]->arBounds[3].x)][int(pNode->pBranches[2]->arBounds[3].y)].vecPos.z;

			// Fourth
			pNode->pBranches[3] = new QTNode;
			pNode->pBranches[3]->pVB = NULL;
			// Init new bounding box
			pNode->pBranches[3]->arBounds[0].x = pNode->arBounds[0].x + uiWidth / 2;
			pNode->pBranches[3]->arBounds[0].y = pNode->arBounds[0].y + uiHeight / 2;			
			pNode->pBranches[3]->arBounds[0].z = m_arTerrain[int(pNode->pBranches[3]->arBounds[0].x)][int(pNode->pBranches[3]->arBounds[0].y)].vecPos.z;
			
			pNode->pBranches[3]->arBounds[1].x = pNode->arBounds[1].x;
			pNode->pBranches[3]->arBounds[1].y = pNode->arBounds[1].y + uiHeight / 2;				
			pNode->pBranches[3]->arBounds[1].z = m_arTerrain[int(pNode->pBranches[3]->arBounds[1].x)][int(pNode->pBranches[3]->arBounds[1].y)].vecPos.z;

			pNode->pBranches[3]->arBounds[2].x = pNode->arBounds[2].x + uiWidth / 2;
			pNode->pBranches[3]->arBounds[2].y = pNode->arBounds[2].y;	
			pNode->pBranches[3]->arBounds[2].z = m_arTerrain[int(pNode->pBranches[2]->arBounds[2].x)][int(pNode->pBranches[3]->arBounds[2].y)].vecPos.z;

			pNode->pBranches[3]->arBounds[3] = pNode->arBounds[3];

			// Create children of the children
			CreateNode(pNode->pBranches[0]);
			CreateNode(pNode->pBranches[1]);
			CreateNode(pNode->pBranches[2]);
			CreateNode(pNode->pBranches[3]);
		}
		return 0;
	}
	return -1;
}

int		CTerrain::FillLeaves()
{
	QTNode *pNode;
	VERTEX *pVertices;
	int v, y, x, i;
	for(i = 0; i < (int)m_arLeaves.size(); i++)
	{
		pNode = (QTNode*)m_arLeaves[i];	
		pNode->pVB->GetBuffer()->Lock(0, 0, (BYTE**) &pVertices, 0);

		v = 0;
		for(y = (int)pNode->arBounds[0].y; y <= (int)pNode->arBounds[2].y; y++)
		{
			for(x = (int)pNode->arBounds[0].x; x <= (int)pNode->arBounds[1].x; x++)
			{
				pVertices[v] = m_arTerrain[x][y];						
				v++;
			}
		}
		pNode->pVB->GetBuffer()->Unlock();
	}
	return -1;
}

int		CTerrain::DestroyNode(QTNode *pNode)
{
	if(pNode)
	{
		if(pNode->ntType == LEAF_TYPE)
		{
			SAFE_RELEASE(pNode->pVB);
		}
		else
		{
			DestroyNode(pNode->pBranches[0]);
			DestroyNode(pNode->pBranches[1]);
			DestroyNode(pNode->pBranches[2]);
			DestroyNode(pNode->pBranches[3]);
		}
		SAFE_DELETE(pNode);
		return 0;
	}
	return -1;
}

int	CTerrain::CircleAlgorithm()
{
	int iCircleX, iCircleY;
	char per[50];
	double pd, circlesize, distance, disp, fPer = 0.0f;
	int	iIterCount = cmnReadSetupInt(_S_TERRAIN, _S_ITERCOUNT);
	int iDisplacement = cmnReadSetupInt(_S_TERRAIN, _S_DISPLACEMENT);
	int iMaxHeight = cmnReadSetupInt(_S_TERRAIN, _S_MAXHEIGHT) * 10;
	//
	// Check terrain validity
	if(!m_arTerrain)
	{
		XException exp("Terrain is not initialized. You cannot apply any algorithm.");
		THROW(exp);
	}
	// reset terrain
	for(int y = 0; y < m_iTerrainVerticesY; y++)
	{
		for(int x = 0; x < m_iTerrainVerticesX; x++)
		{
			m_arTerrain[x][y].vecPos.z = -10.0f;
		}
	}
	for(int i = 0; i < iIterCount; i++)
	{
		// choose random circle center
		iCircleX = rand() % m_iTerrainVerticesX;
		iCircleY = rand() % m_iTerrainVerticesY;

		circlesize = (rand() % iMaxHeight) / 10.0f + 10.0f;
		disp = (rand() % iDisplacement) / 10.0f;

		// apply displacement
		for(int y = 0; y < m_iTerrainVerticesY; y++)
		{
			for(int x = 0; x < m_iTerrainVerticesX; x++)
			{
				// compute distance current point and center point
				distance = sqrt(pow(m_arTerrain[iCircleX][iCircleY].vecPos.x - m_arTerrain[x][y].vecPos.x, 2) + pow(m_arTerrain[iCircleX][iCircleY].vecPos.y - m_arTerrain[x][y].vecPos.y, 2)); 

				pd = distance * 2 / circlesize;

				if(fabs(pd) <= 1.0f)
				{
					m_arTerrain[x][y].vecPos.z += float(disp / 2 + cos(pd * D3DX_PI) * disp / 2);
				}
			}
		}
		IDisplay *pDis;
		CreateDisplayObject(DISIID_IDisplay, (void**) &pDis);

		fPer += (100.0f/iIterCount);
		pDis->UpdateBackground();
		pDis->GetDevice()->BeginScene();
		sprintf(per, "Generating terrain: %3.1lf%%", fPer);
		m_pDebugFont->Draw(per, 0, 50, D3DCOLOR_ARGB(255,255,255,0));
		pDis->GetDevice()->EndScene();
		pDis->Present();

		SAFE_RELEASE(pDis);
	}
	return S_OK;
}
